import { Router } from 'vue-router'
import { ArticleEntry } from '../content/templates/article-list'
import { GalleryEntry } from '../content/templates/gallery-list'
import { TemplateType } from '../content/templates/templateType'
import { WarningModal } from './warnings'

/**
 * Defines the shared options for a route
 */
export type SharedRouteDefinition = {
  id: string
  scriptUrl?: string
  stylesheetUrls: string[]
  template: TemplateType
  title: string
  fullTitle: string
  warning: boolean | WarningModal
}

/**
 * Defines a content-based route
 */
export type ContentfulRouteDefintion = SharedRouteDefinition & {
  content: string
}

/**
 * Defines a config-based route
 */
export type ConfigfulRouteDefinition = SharedRouteDefinition & {
  config: string
}

/**
 * Defines the config for a route using the `markdown` {@link TemplateType}
 */
export type MarkdownDefinition = ContentfulRouteDefintion & {
  template: 'markdown'
}

/**
 * Defines the config for a route using the `article-list` {@link TemplateType}
 */
export type ArticleListDefinition = ConfigfulRouteDefinition & {
  template: 'article-list'
  view: {
    stylesheetUrls: string[]
  }
}

/**
 * Defines the config for a route using the `gallery-list` {@link TemplateType}
 */
export type GalleryListDefinition = ConfigfulRouteDefinition & {
  template: 'gallery-list'
  view: {
    stylesheetUrls: string[]
  }
}

/**
 * Defines all available route definitions
 */
export type RouteDefinition =
  | MarkdownDefinition
  | ArticleListDefinition
  | GalleryListDefinition

/**
 * Defines the collection of routes the app uses
 */
export type RouteCollection = { [key: string]: RouteDefinition }

/**
 * Defines {@link Window} globals
 */
export interface RoutedWindow extends Window {
  /** refers to a template's primary route config; may briefly refer to sub config until a view is fully resolved */
  routeConfig: RouteDefinition
  /** refers to a template's sub config in the case of child routes under a template (e.g. article-view under article-list) */
  routeSubConfig: any
  /** refers to content config for various view routes (e.g. {@link GalleryEntry}, {@link ArticleEntry}, etc.) */
  routeContentConfig: any
  /** vue-router instance */
  router?: Router
}
